==> openbibleviewer_0.8.0.orig.tar.gz: openBibleViewer-0.8.0/src/core/module/bible/thewordbible.cpp <==
/***************************************************************************
openBibleViewer - Bible Study Tool
Copyright (C) 2009-2012 Paul Walger <metaxy@walger.name>
This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the Free
Software Foundation; either version 3 of the License, or (at your option)
any later version.
This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
#include "thewordbible.h"
#include "config.h"
#include "CLucene.h"
#include "CLucene/clucene-config.h"
#include "src/core/search/searchtools.h"
using namespace lucene::analysis;
using namespace lucene::index;
using namespace lucene::queryParser;
using namespace lucene::document;
using namespace lucene::search;
extern bool removeDir(const QString &dirName);
TheWordBible::TheWordBible()
{

}
TheWordBible::~TheWordBible()
{
    m_books.clear();
}

void TheWordBible::setSettings(Settings *set)
{
    m_settings = set;
}
/**
  * Loads the Data.
  */
int TheWordBible::loadData(const int id, const QString &path)
{
    m_book.clear();

    m_moduleID = id;
    m_modulePath = path;
    int dataLines;
    const QString pre = m_settings->homePath + "cache/" + m_settings->hash(m_modulePath) + "/";
    Versification::VersificationFilterFlags flags;
    if(path.endsWith(".nt")) {
        m_settings->getModuleSettings(id)->versificationName = "kjv-nt";
        flags = Versification::ReturnNT;
        dataLines = 7957;
    } else if(path.endsWith(".ot")) {
        m_settings->getModuleSettings(id)->versificationName = "kjv-ot";
        flags = Versification::ReturnOT;
        dataLines = 23145;
    } else if(path.endsWith(".ont")) {
        m_settings->getModuleSettings(id)->versificationName = "kjv";
        flags = Versification::ReturnAll;
        dataLines = 31102;
    } else {
        myWarning() << "wrong file suffix" << path;
        return 1;
    }
    m_versification = QSharedPointer<Versification>(new Versification_KJV());
    m_versification->setFlags(flags);
    myDebug() << flags;
    if(hasHardCache(m_modulePath)) {
        myDebug() << "has already a cache";
        return 0;
    }
    //creating hard cache
    QDir d(pre);
    if(!d.exists())
        d.mkpath(pre);

    QString dataFilePath = path;
    QFile file(dataFilePath);
    if(!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        myWarning() << "could not open file" << dataFilePath;
        return 1;
    }

    myDebug() << m_versification->bookIDs();
    int book = m_versification->bookIDs().first();
    int chapter = 0;
    int verse = 0;
    VerseBook currentBook = VerseBook();
    currentBook.setID(book);
    Chapter currentChapter = Chapter();
    currentChapter.setChapterID(chapter);

    QFile *write = nullptr;
    for(int lineCount = 0; !file.atEnd(); lineCount++) {
        if(lineCount >= dataLines) {
            break;
        }
        if(write == nullptr) {
            write = new QFile(pre + QString::number(book) +".twb");
            write->open(QIODevice::WriteOnly | QIODevice::Text);
            myDebug() << write->fileName();
        }

        write->write(file.readLine());
        if(verse + 1 < m_versification->maxVerse().value(book).at(chapter)) {
            verse++;
        } else {
            if(chapter + 1 < m_versification->maxChapter().value(book)) {
                chapter++;
                verse = 0;
            } else {
                write->close();
                delete write;
                write = nullptr;
                book++;
                chapter = 0;
                verse = 0;
            }
        }
    }

    return 0;

}
VerseBook TheWordBible::loadCached(const int bookID)
{
    DEBUG_FUNC_NAME
    ModuleDisplaySettings *displaySettings = m_settings->getModuleSettings(m_moduleID)->displaySettings().data();
    VerseBook b;
    const QString fileName = m_settings->homePath + "cache/" + m_settings->hash(m_modulePath) + "/" + QString::number(bookID) + ".twb";
    QFile file(fileName);
    if(!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        myWarning() << "file not found " << file.fileName();
        return b;
    }
    QTextStream in(&file);
    int book = bookID;
    int chapter = 0;
    int verse = 0;

    b.setID(book);
    Chapter currentChapter = Chapter();
    currentChapter.setChapterID(chapter);

    for(int lineCount = 0; !in.atEnd(); lineCount++) {
        QString line = in.readLine();
        QRegExp strong("<W(G|H)(\\d+)(x|s)?>");
        QRegExp newLine("<CL>");
        if(displaySettings->showStrong())
            line.replace(strong, "<span class=\"stronglink\"><a href=\""+ModuleTools::strongScheme+"\\1\\2\">\\1\\2</a></span>");
        else
            line.replace(strong, "");

        line.replace(newLine, "<br />");
        line.replace("<CM>", "<br />");
        line.replace("<CI>", "<br />");

        //footnotes
        line.replace("<RF>", "<span class=\"studynote\">");
        line.replace("<Rf>","</span>");

        line.replace("<FU>", "<span class=\"underline\">");
        line.replace("<Fu>","</span>");

        line.replace("<FO>", "<span class=\"otquote\">");
        line.replace("<Fo>","</span>");

        line.replace("<FR>", "<span class=\"wordsofjesus\">");
        line.replace("<Fr>","</span>");

        Verse v(verse, line);
        currentChapter.addVerse(v);
        if(verse + 1 < m_versification->maxVerse().value(book).at(chapter)) {
            verse++;
        } else {
            b.addChapter(currentChapter);
            chapter++;
            verse = 0;

            currentChapter = Chapter();
            currentChapter.setChapterID(chapter);

        }

    }
    return b;
}

bool TheWordBible::hasHardCache(const QString &path)
{
    const QString pre = m_settings->homePath + "cache/" + m_settings->hash(path) + "/";
    ModuleSettings *set = m_settings->getModuleSettings(m_moduleID);
    foreach(int bookID, set->getV11n()->bookIDs()) {
        QFileInfo inf(pre + QString::number(bookID) + ".twb");
        if(!inf.exists() || !inf.isReadable()) {
            myDebug() << "cache not existent at " <<inf.fileName();
            return false;
        }
    }
    return true;
}

int TheWordBible::readBook(const int id)
{
    DEBUG_FUNC_NAME
    m_bookID = id;
    m_book = loadCached(m_bookID);
    return 0;
}

MetaInfo TheWordBible::readInfo(QFile &file)
{
    MetaInfo info;
    bool skipping = true;
    QTextStream in(&file);
    int linesToSkip;
    if(file.fileName().endsWith(".ont")) {
        linesToSkip = 31102;
    } else if(file.fileName().endsWith(".ot")) {
        linesToSkip = 23145;
    } else if(file.fileName().endsWith(".nt")) {
        linesToSkip = 7957;
    } else {
        return info;
    }
    for(int lineCount = 0; !in.atEnd(); lineCount++) {
        const QString line = in.readLine();
        if(!skipping) {
            if(line.startsWith("title")) {
                const QStringList list = line.split("=");
                info.setName(list.last());
            } else if(line.startsWith("short.title")) {
                const QStringList list = line.split("=");
                info.setShortName(list.last());
            } else if(line.startsWith("description")) {
                const QStringList list = line.split("=");
                info.description = list.last();
            } else if(line.startsWith("lang")) {
                const QStringList list = line.split("=");
                info.language = list.last();
            }
        }
        if(line.isEmpty() || lineCount >= linesToSkip)
            skipping = false;

    }
    if(file.fileName().endsWith(".ot")) {
        info.setContent(ModuleTools::BibleOTContent);
    } else if(file.fileName().endsWith(".nt")) {
        info.setContent(ModuleTools::BibleNTContent);
    } else {
        info.setContent(ModuleTools::BibleContent);
    }
    info.setDefaultModule(ModuleTools::DefaultBibleModule);
    /*
            if(line.startsWith("title")) {
                const QStringList list = line.split("=");
                m_moduleName = list.last();
            } else if(line.startsWith("short.title")) {
                const QStringList list = line.split("=");
                m_shortModuleName = list.last();
            } else if(line.startsWith("id")) {
                const QStringList list = line.split("=");
                m_uID = list.last();
            }*/
    return info;
}
MetaInfo TheWordBible::readInfo(const QString &fileName)
{
    QFile file(fileName);
    if(!file.open(QIODevice::ReadOnly))
        return MetaInfo();
    return readInfo(file);
}

void TheWordBible::search(const SearchQuery &query, SearchResult *res) const
{
    const QString index = indexPath();
    const TCHAR* stop_words[] = { nullptr };
    standard::StandardAnalyzer analyzer(stop_words);
    IndexReader* reader = IndexReader::open(index.toStdString().c_str());
    IndexSearcher s(reader);
    Query* q = QueryParser::parse(SearchTools::toTCHAR(query.searchText), _T("content"), &analyzer);
    Hits* h = s.search(q);
    for(size_t i = 0; i < h->length(); i++) {
        Document* doc = &h->doc(i);
        const QString stelle = SearchTools::toQString(doc->get(_T("key")));

        const QStringList l = stelle.split(";");
        if(query.range == SearchQuery::Whole || (query.range == SearchQuery::OT && l.at(0).toInt() <= 38) || (query.range == SearchQuery::NT && l.at(0).toInt() > 38)) {
            SearchHit hit;
            hit.setType(SearchHit::BibleHit);
            hit.setValue(SearchHit::ModuleID, m_moduleID);
            hit.setValue(SearchHit::BookID, l.at(0).toInt());
            hit.setValue(SearchHit::ChapterID, l.at(1).toInt());
            hit.setValue(SearchHit::VerseID, l.at(2).toInt());

            hit.setValue(SearchHit::VerseText, SearchTools::toQString(doc->get(_T("content"))));

            hit.setValue(SearchHit::Score, (double) h->score(i));

            res->addHit(hit);
        }
    }
    reader->close();
    delete reader;
}
bool TheWordBible::hasIndex() const
{
    const QString index = indexPath();
    QDir d;
    if(!d.exists(index)) {
        return false;
    }
    return IndexReader::indexExists(index.toStdString().c_str());
}
void TheWordBible::buildIndex()
{
    QProgressDialog progress(QObject::tr("Build index"), QObject::tr("Cancel"), 0, m_versification->bookIDs().size());
    const QString index = indexPath();
    QDir dir("/");
    dir.mkpath(index);

    IndexWriter* writer = nullptr;
    const TCHAR* stop_words[] = { nullptr };
    standard::StandardAnalyzer an(stop_words);
    if(IndexReader::indexExists(index.toStdString().c_str())) {
        if(IndexReader::isLocked(index.toStdString().c_str())) {
            myDebug() << "Index was locked... unlocking it.";
            IndexReader::unlock(index.toStdString().c_str());
        }
    }
    writer = new IndexWriter(index.toStdString().c_str() , &an, true);

    writer->setMaxFieldLength(0x7FFFFFFFL);
    writer->setUseCompoundFile(false);

    //index
    Document doc;
    bool canceled = false;
    int c = 0;
    auto buffer = SearchTools::createBuffer();
    foreach(const int bookID, m_versification->bookIDs()) {
        c++;
        if(progress.wasCanceled()) {
            canceled = true;
            break;
        }
        progress.setValue(c);
        VerseBook book = loadCached(bookID);
        QHashIterator<int, Chapter> chapterIt(book.data());
        while(chapterIt.hasNext()) {
            chapterIt.next();
            QMapIterator<int, Verse> verseIt(chapterIt.value().data());
            while(verseIt.hasNext()) {
                verseIt.next();
                doc.clear();
                const QString key = QString::number(book.bookID()) + ";" + QString::number(chapterIt.value().chapterID()) + ";" + QString::number(verseIt.value().verseID());
                const QString text = verseIt.value().data();
                SearchTools::toTCHAR(key, buffer);
                doc.add(*new Field(_T("key"), buffer, Field::STORE_YES |  Field::INDEX_NO));
                SearchTools::toTCHAR(text, buffer);
                doc.add(*new Field(_T("content"), buffer, Field::STORE_YES |  Field::INDEX_TOKENIZED));
                writer->addDocument(&doc);

            }
        }
    }
    if(canceled) {
        ModuleTools::removeDir(index);
    } else {
        writer->setUseCompoundFile(true);
        writer->optimize();
    }

    writer->close();
    delete writer;
    delete[] buffer;
    progress.close();
}

int TheWordBible::moduleID() const
{
    return m_moduleID;
}

QString TheWordBible::modulePath() const
{
    return m_modulePath;
}
QString TheWordBible::moduleName(bool preferShortName) const
{
    if(preferShortName) {
        if(!m_shortModuleName.isEmpty()) {
            return m_shortModuleName;
        } else {
            return m_moduleName;
        }
    } else {
        if(!m_moduleName.isEmpty()) {
            return m_moduleName;
        } else {
            return m_shortModuleName;
        }
    }
}

bool TheWordBible::hasNT() const
{
    QFileInfo f(m_modulePath + ".nt");
    return f.exists();
}
bool TheWordBible::hasOT() const
{
    QFileInfo f(m_modulePath + ".ot");
    return f.exists();
}
bool TheWordBible::hasONT() const
{
    QFileInfo f(m_modulePath + ".ont");
    return f.exists();
}
/**
  * Returns the path, where all indexed files are stored.
  */
QString TheWordBible::indexPath() const
{
    return m_settings->homePath + "index/" + m_settings->hash(m_modulePath);
}
QString TheWordBible::uid() const
{
    return m_uID;
}
TextRange TheWordBible::rawTextRange(int bookID, int chapterID, int startVerse, int endVerse)
{
    TextRange ret;
    if(m_book.bookID() != bookID) {
        myDebug() << "book not loaded";
        readBook(bookID);
    }
    if(!m_book.hasChapter(chapterID)) {
        myWarning() << "index out of range index chapterID = " << chapterID;
        return ret;
    }
    ret.setModuleID(m_moduleID);
    ret.setBookID(bookID);
    ret.setChapterID(chapterID);

    const Chapter c = m_book.getChapter(chapterID);
    QMap<int, Verse> data = c.data();
    QMapIterator<int, Verse> i(data);
    while(i.hasNext()) {
        i.next();
        myDebug() << i.key();
        if((i.key() <= endVerse || endVerse == -1) && //if endverse == -1 this means no filtering
            (i.key() >= startVerse || startVerse == -1))
            ret.addVerse(i.value());
    }
    return ret;
}

std::pair<int, int> TheWordBible::minMaxVerse(int bookID, int chapterID)
{
    std::pair<int, int> ret;

    if(m_book.bookID() != bookID) {
        readBook(bookID);
        myDebug() << "book not loaded";
    }
    if(!m_book.hasChapter(chapterID)) {
        myWarning() << "index out of range index chapterID = " << chapterID;
        return ret;
    }

    const Chapter c = m_book.getChapter(chapterID);
    //because c.data() is a map and it is sorted by key
    ret.first = c.data().keys().first();
    ret.second = c.data().keys().last();
    myDebug() << ret.first << ret.second;
    return ret;
}

void TheWordBible::clearData()
{
    m_books.clear();
}

==> openbibleviewer_0.9.0.orig.tar.gz: openBibleViewer-0.9.0/src/core/module/bible/thewordbible.cpp <==
/***************************************************************************
openBibleViewer - Bible Study Tool
Copyright (C) 2009-2012 Paul Walger <metaxy@walger.name>
This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the Free
Software Foundation; either version 3 of the License, or (at your option)
any later version.
This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
#include "thewordbible.h"
#include "config.h"
#include "CLucene.h"
#include "CLucene/clucene-config.h"
#include "src/core/search/searchtools.h"
using namespace lucene::analysis;
using namespace lucene::index;
using namespace lucene::queryParser;
using namespace lucene::document;
using namespace lucene::search;
extern bool removeDir(const QString &dirName);
TheWordBible::TheWordBible()
{

}
TheWordBible::~TheWordBible()
{
    m_books.clear();
}

void TheWordBible::setSettings(Settings *set)
{
    m_settings = set;
}
/**
  * Loads the Data.
  */
int TheWordBible::loadData(const int id, const QString &path)
{
    m_book.clear();

    m_moduleID = id;
    m_modulePath = path;
    int dataLines;
    const QString pre = m_settings->homePath + "cache/" + m_settings->hash(m_modulePath) + "/";
    Versification::VersificationFilterFlags flags;
    if(path.endsWith(".nt")) {
        m_settings->getModuleSettings(id)->versificationName = "kjv-nt";
        flags = Versification::ReturnNT;
        dataLines = 7957;
    } else if(path.endsWith(".ot")) {
        m_settings->getModuleSettings(id)->versificationName = "kjv-ot";
        flags = Versification::ReturnOT;
        dataLines = 23145;
    } else if(path.endsWith(".ont")) {
        m_settings->getModuleSettings(id)->versificationName = "kjv";
        flags = Versification::ReturnAll;
        dataLines = 31102;
    } else {
        myWarning() << "wrong file suffix" << path;
        return 1;
    }
    m_versification = QSharedPointer<Versification>(new Versification_KJV());
    m_versification->setFlags(flags);
    myDebug() << flags;
    if(hasHardCache(m_modulePath)) {
        myDebug() << "has already a cache";
        return 0;
    }
    //creating hard cache
    QDir d(pre);
    if(!d.exists())
        d.mkpath(pre);

    QString dataFilePath = path;
    QFile file(dataFilePath);
    if(!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        myWarning() << "could not open file" << dataFilePath;
        return 1;
    }

    myDebug() << m_versification->bookIDs();
    int book = m_versification->bookIDs().first();
    int chapter = 0;
    int verse = 0;
    VerseBook currentBook = VerseBook();
    currentBook.setID(book);
    Chapter currentChapter = Chapter();
    currentChapter.setChapterID(chapter);

    QFile *write = nullptr;
    for(int lineCount = 0; !file.atEnd(); lineCount++) {
        if(lineCount >= dataLines) {
            break;
        }
        if(write == nullptr) {
            write = new QFile(pre + QString::number(book) +".twb");
            write->open(QIODevice::WriteOnly | QIODevice::Text);
            myDebug() << write->fileName();
        }

        write->write(file.readLine());
        if(verse + 1 < m_versification->maxVerse().value(book).at(chapter)) {
            verse++;
        } else {
            if(chapter + 1 < m_versification->maxChapter().value(book)) {
                chapter++;
                verse = 0;
            } else {
                write->close();
                delete write;
                write = nullptr;
                book++;
                chapter = 0;
                verse = 0;
            }
        }
    }

    return 0;

}
VerseBook TheWordBible::loadCached(const int bookID)
{
    DEBUG_FUNC_NAME
    ModuleDisplaySettings *displaySettings = m_settings->getModuleSettings(m_moduleID)->displaySettings().data();
    VerseBook b;
    const QString fileName = m_settings->homePath + "cache/" + m_settings->hash(m_modulePath) + "/" + QString::number(bookID) + ".twb";
    QFile file(fileName);
    if(!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        myWarning() << "file not found " << file.fileName();
        return b;
    }
    QTextStream in(&file);
    int book = bookID;
    int chapter = 0;
    int verse = 0;

    b.setID(book);
    Chapter currentChapter = Chapter();
    currentChapter.setChapterID(chapter);

    for(int lineCount = 0; !in.atEnd(); lineCount++) {
        QString line = in.readLine();
        QRegExp strong("<W(G|H)(\\d+)(x|s)?>");
        QRegExp newLine("<CL>");
        if(displaySettings->showStrong())
            line.replace(strong, "<span class=\"stronglink\"><a href=\""+ModuleTools::strongScheme+"\\1\\2\">\\1\\2</a></span>");
        else
            line.replace(strong, "");

        line.replace(newLine, "<br />");
        line.replace("<CM>", "<br />");
        line.replace("<CI>", "<br />");

        //footnotes
        line.replace("<RF>", "<span class=\"studynote\">");
        line.replace("<Rf>","</span>");

        line.replace("<FU>", "<span class=\"underline\">");
        line.replace("<Fu>","</span>");

        line.replace("<FO>", "<span class=\"otquote\">");
        line.replace("<Fo>","</span>");

        line.replace("<FR>", "<span class=\"wordsofjesus\">");
        line.replace("<Fr>","</span>");

        Verse v(verse, line);
        currentChapter.addVerse(v);
        if(verse + 1 < m_versification->maxVerse().value(book).at(chapter)) {
            verse++;
        } else {
            b.addChapter(currentChapter);
            chapter++;
            verse = 0;

            currentChapter = Chapter();
            currentChapter.setChapterID(chapter);

        }

    }
    return b;
}

bool TheWordBible::hasHardCache(const QString &path)
{
    const QString pre = m_settings->homePath + "cache/" + m_settings->hash(path) + "/";
    ModuleSettings *set = m_settings->getModuleSettings(m_moduleID);
    foreach(int bookID, set->getV11n()->bookIDs()) {
        QFileInfo inf(pre + QString::number(bookID) + ".twb");
        if(!inf.exists() || !inf.isReadable()) {
            myDebug() << "cache not existent at " <<inf.fileName();
            return false;
        }
    }
    return true;
}

int TheWordBible::readBook(const int id)
{
    DEBUG_FUNC_NAME
    m_bookID = id;
    m_book = loadCached(m_bookID);
    return 0;
}

MetaInfo TheWordBible::readInfo(QFile &file)
{
    MetaInfo info;
    bool skipping = true;
    QTextStream in(&file);
    int linesToSkip;
    if(file.fileName().endsWith(".ont")) {
        linesToSkip = 31102;
    } else if(file.fileName().endsWith(".ot")) {
        linesToSkip = 23145;
    } else if(file.fileName().endsWith(".nt")) {
        linesToSkip = 7957;
    } else {
        return info;
    }
    for(int lineCount = 0; !in.atEnd(); lineCount++) {
        const QString line = in.readLine();
        if(!skipping) {
            if(line.startsWith("title")) {
                const QStringList list = line.split("=");
                info.setName(list.last());
            } else if(line.startsWith("short.title")) {
                const QStringList list = line.split("=");
                info.setShortName(list.last());
            } else if(line.startsWith("description")) {
                const QStringList list = line.split("=");
                info.description = list.last();
            } else if(line.startsWith("lang")) {
                const QStringList list = line.split("=");
                info.language = list.last();
            }
        }
        if(line.isEmpty() || lineCount >= linesToSkip)
            skipping = false;

    }
    if(file.fileName().endsWith(".ot")) {
        info.setContent(ModuleTools::BibleOTContent);
    } else if(file.fileName().endsWith(".nt")) {
        info.setContent(ModuleTools::BibleNTContent);
    } else {
        info.setContent(ModuleTools::BibleContent);
    }
    info.setDefaultModule(ModuleTools::DefaultBibleModule);
    /*
            if(line.startsWith("title")) {
                const QStringList list = line.split("=");
                m_moduleName = list.last();
            } else if(line.startsWith("short.title")) {
                const QStringList list = line.split("=");
                m_shortModuleName = list.last();
            } else if(line.startsWith("id")) {
                const QStringList list = line.split("=");
                m_uID = list.last();
            }*/
    return info;
}
MetaInfo TheWordBible::readInfo(const QString &fileName)
{
    QFile file(fileName);
    if(!file.open(QIODevice::ReadOnly))
        return MetaInfo();
    return readInfo(file);
}

void TheWordBible::search(const SearchQuery &query, SearchResult *res) const
{
    const QString index = indexPath();
    const TCHAR* stop_words[] = { nullptr };
    standard::StandardAnalyzer analyzer(stop_words);
    IndexReader* reader = IndexReader::open(index.toStdString().c_str());
    IndexSearcher s(reader);
    Query* q = QueryParser::parse(SearchTools::toTCHAR(query.searchText), _T("content"), &analyzer);
    Hits* h = s.search(q);
    for(size_t i = 0; i < h->length(); i++) {
        Document* doc = &h->doc(i);
        const QString stelle = SearchTools::toQString(doc->get(_T("key")));

        const QStringList l = stelle.split(";");
        if(query.range == SearchQuery::Whole || (query.range == SearchQuery::OT && l.at(0).toInt() <= 38) || (query.range == SearchQuery::NT && l.at(0).toInt() > 38)) {
            SearchHit hit;
            hit.setType(SearchHit::BibleHit);
            hit.setValue(SearchHit::ModuleID, m_moduleID);
            hit.setValue(SearchHit::BookID, l.at(0).toInt());
            hit.setValue(SearchHit::ChapterID, l.at(1).toInt());
            hit.setValue(SearchHit::VerseID, l.at(2).toInt());

            hit.setValue(SearchHit::VerseText, SearchTools::toQString(doc->get(_T("content"))));

            hit.setValue(SearchHit::Score, (double) h->score(i));

            res->addHit(hit);
        }
    }
    reader->close();
    delete reader;
}
bool TheWordBible::hasIndex() const
{
    const QString index = indexPath();
    QDir d;
    if(!d.exists(index)) {
        return false;
    }
    return IndexReader::indexExists(index.toStdString().c_str());
}
void TheWordBible::buildIndex()
{
    QProgressDialog progress(QObject::tr("Build index"), QObject::tr("Cancel"), 0, m_versification->bookIDs().size());
    const QString index = indexPath();
    QDir dir("/");
    dir.mkpath(index);

    IndexWriter* writer = nullptr;
    const TCHAR* stop_words[] = { nullptr };
    standard::StandardAnalyzer an(stop_words);
    if(IndexReader::indexExists(index.toStdString().c_str())) {
        if(IndexReader::isLocked(index.toStdString().c_str())) {
            myDebug() << "Index was locked... unlocking it.";
            IndexReader::unlock(index.toStdString().c_str());
        }
    }
    writer = new IndexWriter(index.toStdString().c_str() , &an, true);

    writer->setMaxFieldLength(0x7FFFFFFFL);
    writer->setUseCompoundFile(false);

    //index
    Document doc;
    bool canceled = false;
    int c = 0;
    auto buffer = SearchTools::createBuffer();
    foreach(const int bookID, m_versification->bookIDs()) {
        c++;
        if(progress.wasCanceled()) {
            canceled = true;
            break;
        }
        progress.setValue(c);
        VerseBook book = loadCached(bookID);
        QHashIterator<int, Chapter> chapterIt(book.data());
        while(chapterIt.hasNext()) {
            chapterIt.next();
            QMapIterator<int, Verse> verseIt(chapterIt.value().data());
            while(verseIt.hasNext()) {
                verseIt.next();
                doc.clear();
                const QString key = QString::number(book.bookID()) + ";" + QString::number(chapterIt.value().chapterID()) + ";" + QString::number(verseIt.value().verseID());
                const QString text = verseIt.value().data();
                SearchTools::toTCHAR(key, buffer);
                doc.add(*new Field(_T("key"), buffer, Field::STORE_YES |  Field::INDEX_NO));
                SearchTools::toTCHAR(text, buffer);
                doc.add(*new Field(_T("content"), buffer, Field::STORE_YES |  Field::INDEX_TOKENIZED));
                writer->addDocument(&doc);

            }
        }
    }
    if(canceled) {
        ModuleTools::removeDir(index);
    } else {
        writer->setUseCompoundFile(true);
        writer->optimize();
    }

    writer->close();
    delete writer;
    delete[] buffer;
    progress.close();
}

int TheWordBible::moduleID() const
{
    return m_moduleID;
}

QString TheWordBible::modulePath() const
{
    return m_modulePath;
}
QString TheWordBible::moduleName(bool preferShortName) const
{
    if(preferShortName) {
        if(!m_shortModuleName.isEmpty()) {
            return m_shortModuleName;
        } else {
            return m_moduleName;
        }
    } else {
        if(!m_moduleName.isEmpty()) {
            return m_moduleName;
        } else {
            return m_shortModuleName;
        }
    }
}

bool TheWordBible::hasNT() const
{
    QFileInfo f(m_modulePath + ".nt");
    return f.exists();
}
bool TheWordBible::hasOT() const
{
    QFileInfo f(m_modulePath + ".ot");
    return f.exists();
}
bool TheWordBible::hasONT() const
{
    QFileInfo f(m_modulePath + ".ont");
    return f.exists();
}
/**
  * Returns the path, where all indexed files are stored.
  */
QString TheWordBible::indexPath() const
{
    return m_settings->homePath + "index/" + m_settings->hash(m_modulePath);
}
QString TheWordBible::uid() const
{
    return m_uID;
}
TextRange TheWordBible::rawTextRange(int bookID, int chapterID, int startVerse, int endVerse)
{
    TextRange ret;
    if(m_book.bookID() != bookID) {
        myDebug() << "book not loaded";
        readBook(bookID);
    }
    if(!m_book.hasChapter(chapterID)) {
        myWarning() << "index out of range index chapterID = " << chapterID;
        return ret;
    }
    ret.setModuleID(m_moduleID);
    ret.setBookID(bookID);
    ret.setChapterID(chapterID);

    const Chapter c = m_book.getChapter(chapterID);
    QMap<int, Verse> data = c.data();
    QMapIterator<int, Verse> i(data);
    while(i.hasNext()) {
        i.next();
        myDebug() << i.key();
        if((i.key() <= endVerse || endVerse == -1) && //if endverse == -1 this means no filtering
            (i.key() >= startVerse || startVerse == -1))
            ret.addVerse(i.value());
    }
    return ret;
}

std::pair<int, int> TheWordBible::minMaxVerse(const int bookID, const int chapterID)
{
    std::pair<int, int> ret;

    if(m_book.bookID() != bookID) {
        readBook(bookID);
        myDebug() << "book not loaded";
    }
    if(!m_book.hasChapter(chapterID)) {
        myWarning() << "index out of range index chapterID = " << chapterID;
        return ret;
    }

    const Chapter c = m_book.getChapter(chapterID);
    //because c.data() is a map and it is sorted by key
    ret.first = c.data().keys().first();
    ret.second = c.data().keys().last();
    myDebug() << ret.first << ret.second;
    return ret;
}

void TheWordBible::clearData()
{
    m_books.clear();
}
